package top.ninwoo.hadoop.client.service;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import top.ninwoo.hadoop.api.po.ComputingResult;
import top.ninwoo.hadoop.api.po.FileSlice;
import top.ninwoo.hadoop.api.po.MergeResult;
import top.ninwoo.hadoop.api.po.SplitResult;
import top.ninwoo.hadoop.api.service.HadoopComService;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import top.ninwoo.bishe.starter.service.NetworkService;

/**
 * 文件服务
 */
@Service
public class FileService{


    @Value("${dbc.directory.output}")
    private String directoryOutput;

    @Qualifier(value="fileRestTemplates")
    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private NetworkService networkService;

    @Autowired
    private HadoopComService hadoopComService;

    Long clusterId = 11113L;//docker集群ID

    String[] wxName_list = new String[]{"sate1","sate2","sate3","sate4","sate5","sate6","sate7","sate8","sate9","sate10","sate11",
            "sate12","sate13","sate14","sate15","sate16","sate17","sate18","sate19","sate20","sate21",
            "sate22","sate23","sate24","sate25","sate26","sate27","sate28","sate29"};//获取卫星名字
    private String[] ipList = new String[29];
    private List <Integer> index = new ArrayList<Integer>() {} ;


    public String getIpByAppName(Long clusterId,String appName) {
        List<String> ipList = networkService.getIpListByAppName(clusterId, appName);
        if(!ipList.isEmpty()){
            String ip_tmp = ipList.get(0);
            String[] split_list = ip_tmp.split("/");
            return split_list[0] + ":8082";
        }
        return null;
    }

    /**
     * 发送文件到分布式节点
     * @param fileId
     * @param bufferedImage
     * @return
     */

    long startTime = System.currentTimeMillis()/1000; // 获取开始时间
    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//设置日期格式

    @Async("taskExecutor")
    public String sendFile(Long fileId, BufferedImage bufferedImage) {

//        System.out.println("-----------------------------The time of sending file--------------------------------");
//        long startTime = System.currentTimeMillis()/1000; // 获取开始时间
        System.out.println("The current system time at the beginning of the program:" +df.format(new Date()));// new Date()为获取当前系统时间

        for (int i = 0; i < wxName_list.length; i++){
            ipList[i] = getIpByAppName(clusterId,wxName_list[i]);//通过卫星名字获取卫星ip
        }
        //查看ip
        System.out.println(ipList.length);
        System.out.println(Arrays.toString(ipList));

        index = this.getIndex(ipList.length);
        int sliceNum = 5;

        // todo 这里需要制定文件的fileId
        SplitResult splitResult = hadoopComService.fileSplit(bufferedImage, sliceNum);
        int i = 0;
        for (FileSlice slice : splitResult.getFileSliceList()) {
            slice.setFileId(fileId);
            //异常处理
            while(true){
                try {
                    sendFileSlice(ipList[index.get(i)], slice);
                    System.out.println("send "+wxName_list[index.get(i)]+" success");
                    i = i+1;
                    break;
                }catch (Exception e){
                    System.out.println("send "+wxName_list[index.get(i)]+" failed");
                }
            }
        }

       /* long endTime = System.currentTimeMillis()/1000; //获取程序结束时间
        System.out.println("The current system time at the end of the program:"+ df.format(new Date()));// new Date()为获取当前系统时间
        System.out.println("The total running time of the program:" + (endTime - startTime) + "s");
        System.out.println("=====================================================================================");*/

        return "success";

    }

    // 发送文件切片到目标地址
    public String sendFileSlice(String targetIp, FileSlice fileSlice) {
        ResponseEntity<String> response = restTemplate.postForEntity("http://" + targetIp + "/data/put/", fileSlice, String.class);
        if (!response.getStatusCode().is2xxSuccessful()) {
            return "failed!";
        }
        return response.getBody();
    }


    public Set<FileSlice> getFileSlice(String targetIp, Long fileId) {
        FileSlice[] fileSlices = restTemplate.getForObject("http://" + targetIp + "/data/get/" + fileId, FileSlice[].class);
        HashSet<FileSlice> fileSliceSet = new HashSet<>();
        fileSliceSet.addAll(Arrays.asList(fileSlices));
        return fileSliceSet;
    }

    public String getFile(String fileName, Long fileId) {

/*        System.out.println("--------------------The time of geting file------------------------------------------");
        long startTime1 = System.currentTimeMillis()/1000; // 获取开始时间
        System.out.println("The current system time at the beginning of the program:" +df.format(new Date()));*/

        for (int i = 0; i < wxName_list.length; i++){
            ipList[i] = getIpByAppName(clusterId,wxName_list[i]);//通过卫星名字获取卫星ip
        }
        int sliceNum = 5;
        Set<FileSlice> result = new HashSet<>();
        //异常处理
        for (int k = 0; k < 5; k++) {
            try {
                Set<FileSlice> fileSliceSet = getFileSlice(ipList[index.get(k)], fileId);
                System.out.println("recive "+wxName_list[index.get(k)]+" success");
//                Set<FileSlice> fileSliceSet = getFileSlice(ipList[k], fileId);
//                System.out.println("recive "+wxName_list[k]+" success");
                result.addAll(fileSliceSet);
            }catch (Exception e){
                System.out.println("recive fileslice fail");
            }
        }
        List<FileSlice> list = new ArrayList<>(result);
        SplitResult splitResult = new SplitResult();
        splitResult.setFileSliceList(sortList(list));

        //merge
        ComputingResult computingResult = hadoopComService.sliceComputing(splitResult);
        MergeResult mergeResult = hadoopComService.sliceMerge(computingResult,sliceNum);
        //save
        try {
            saveFile(fileName, mergeResult);
            long endTime = System.currentTimeMillis()/1000; //获取程序结束时间
            System.out.println("The current system time at the end of the program:"+ df.format(new Date()));// new Date()为获取当前系统时间
            System.out.println("The total running time of the program:" + (endTime - startTime) + "s");
            return "success";
        } catch (Exception e) {
            e.printStackTrace();
            return "fail";
        }
    }

    //给List集合里的元素进行排序
    public List<FileSlice> sortList(List<FileSlice> list){
        for (int i = 0; i < list.size(); i++)    {
            for (int j = list .size()-1; j > i; j--)  {
                int no= list.get(j).getSliceId();
                int no_1= list.get(j-1).getSliceId();
                if (no<no_1)    {
                    //互换位置
                    FileSlice  fil = list.get(j);
                    list.set(j, list.get(j-1));
                    list.set(j-1, fil );
                }
            }
        }
        return list;
    }

    public void saveFile(String fileName, MergeResult mergeResult) {
        BufferedImage images = mergeResult.getImages();
        //输出拼接后的图像
        try {
            ImageIO.write(images, "jpg", new File(directoryOutput + fileName));
        } catch (IOException e) {
            e.printStackTrace();
        }
/*        long endTime = System.currentTimeMillis(); //获取程序结束时间
        System.out.println("The current system time at the end of the program:"+ df.format(new Date()));// new Date()为获取当前系统时间
        System.out.println("The execution time at the end of the program:"+ endTime);
        System.out.println("The total running time of the program:" + (endTime - startTime) + "ms");*/

    }
    public List <Integer> getIndex(int num){
        Random r = new Random();
        List <Integer> v = new ArrayList<Integer>() {} ;

        int count = 0;
        while(count < 5){
            int number = r.nextInt(num) + 1;
            if(!v.contains(number)){
                //不在集合中，就添加
                v.add(number);
                count++;
            }
        }
        return v;
    }

}
